/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2004 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */


#include <Socket.h>
#include "TestSocket.h"
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <iostream>
#include <errno.h>
#include <syslog.h>


TestSocket::TestSocket():
		  Socket()
{
	 set_so_reuseaddr(1);
	 _buf=new u_int8_t[1024];
}

TestSocket::TestSocket(const int& m):
		  Socket(m)
{
	 set_so_reuseaddr(1);
	 _buf=new u_int8_t[_mtu];
}

TestSocket::TestSocket(const TestSocket &s):
		  Socket()
{
    _sockd=s._sockd;
	 _mtu=s._mtu;
	 _buf=new u_int8_t[_mtu];
	 set_so_reuseaddr(1);
}

TestSocket::~TestSocket()
{
    // This is an important feature of the TestSocket class. Especially
    // for copying of TestSockets. TestSockets do *not* close their
	 // sockets when they are destroyed.
	 syslog(LOG_ERR, "TestSocket::~TestSocket(): _bytes_sent==%lld", _bytes_sent);
	 syslog(LOG_ERR, "TestSocket::~TestSocket(): _bytes_rcvd==%lld", _bytes_rcvd);
	 syslog(LOG_ERR, "TestSocket::~TestSocket(): _send_calls==%lld", _send_calls);
	 syslog(LOG_ERR, "TestSocket::~TestSocket(): _recv_calls==%lld", _recv_calls);
	 _age.stop();
	 double duration=_age.elapsed();
	 double rcv_rate=_bytes_rcvd/duration*8.0/1000000;
	 double send_rate=_bytes_sent/duration*8.0/1000000;
	 syslog(LOG_ERR, "TestSocket::~TestSocket(): duration==%f", duration);
	 syslog(LOG_ERR, "TestSocket::~TestSocket(): rcv_rate==%f", rcv_rate);
	 syslog(LOG_ERR, "TestSocket::~TestSocket(): send_rate==%f", send_rate);
	 delete [] _buf;
}

TestSocket& TestSocket::operator=(const TestSocket& s)
{
    _sockd=s._sockd;
	 _mtu=s._mtu;
	 set_so_reuseaddr(1);
    return(*this);
}

void TestSocket::set_sockd(const int& s)
{
    if (_sockd!=-1)
        ::close(_sockd);
    _sockd=s;
}

void TestSocket::set_mtu(const int& m)
{
	 _mtu=m;
}

int TestSocket::bind(const string& ip, const int& port)
{
	 return(0);
}

int TestSocket::listen()
{
	 return(0);
}

int TestSocket::accept()
{
    return(1);
}

int TestSocket::connect(const string& ip, const int& port)
{
	 return(0);
}

int TestSocket::close()
{
	 return(0);
}

int TestSocket::shutdown(const int& howto)
{
	 return(0);
}

ostream& operator<<(ostream& os, const TestSocket& s)
{
	 os << "TestSocket dump\n";
    os << "    _sockd:         " << s.get_sockd() << endl;
    os << "    _bytes_sent:    " << s.get_bytes_sent() << endl;
    os << "    _bytes_rcvd:    " << s.get_bytes_rcvd() << endl;
    os << "    _send_calls:    " << s.get_send_calls() << endl;
    os << "    _recv_calls:    " << s.get_recv_calls() << endl;
	 os << "-TestSocket dump\n";
	 os << "TestSock dump\n";
    return(os);
}

